@@ -28,7 +28,8 @@
 #include <QUuid>
 #include "bufferadaptor.h"
 
-#define SINK_ENTITY(TYPE) \
+#define SINK_ENTITY(TYPE, LOWERCASENAME) \
+    static constexpr const char *name = #LOWERCASENAME; \
     typedef QSharedPointer<TYPE> Ptr; \
     using Entity::Entity; \
     TYPE() = default; \
@@ -92,6 +93,43 @@
 namespace Sink {
 namespace ApplicationDomain {
 
+enum SINK_EXPORT ErrorCode {
+    NoError = 0,
+    UnknownError,
+    NoServerError,
+    ConnectionError,
+    LoginError,
+    ConfigurationError,
+    TransmissionError,
+};
+
+enum SINK_EXPORT SuccessCode {
+    TransmissionSuccess
+};
+
+enum SINK_EXPORT SyncStatus {
+    NoSyncStatus,
+    SyncInProgress,
+    SyncError,
+    SyncSuccess
+};
+
+/**
+ * The status of an account or resource.
+ *
+ * It is set as follows:
+ * * By default the status is offline.
+ * * If a connection to the server could be established the status is Connected.
+ * * If an error occurred that keeps the resource from operating (so non transient), the resource enters the error state.
+ * * If a long running operation is started the resource goes to the busy state (and return to the previous state after that).
+ */
+enum SINK_EXPORT Status {
+    OfflineStatus,
+    ConnectedStatus,
+    BusyStatus,
+    ErrorStatus
+};
+
 struct SINK_EXPORT Error {
 
 };
@@ -100,6 +138,11 @@ struct SINK_EXPORT Progress {
 
 };
 
+/**
+ * Internal type.
+ *
+ * Represents a BLOB property.
+ */
 struct BLOB {
     BLOB() = default;
     BLOB(const BLOB &) = default;
@@ -268,6 +311,7 @@ SINK_EXPORT QDebug operator<< (QDebug d, const BLOB &blob);
 
 
 struct SINK_EXPORT SinkAccount : public ApplicationDomainType {
+    static constexpr const char *name = "account";
     typedef QSharedPointer<SinkAccount> Ptr;
     explicit SinkAccount(const QByteArray &resourceInstanceIdentifier, const QByteArray &identifier, qint64 revision, const QSharedPointer<BufferAdaptor> &adaptor);
     explicit SinkAccount(const QByteArray &identifier);
@@ -278,8 +322,6 @@ struct SINK_EXPORT SinkAccount : public ApplicationDomainType {
     SINK_PROPERTY(QString, Icon, icon);
     SINK_PROPERTY(QString, AccountType, type);
     SINK_STATUS_PROPERTY(int, Status, status);
-    SINK_STATUS_PROPERTY(ApplicationDomain::Error, Error, error);
-    SINK_STATUS_PROPERTY(ApplicationDomain::Progress, Progress, progress);
 };
 
 
@@ -290,6 +332,7 @@ struct SINK_EXPORT SinkAccount : public ApplicationDomainType {
  * and for creating and removing resource instances.
  */
 struct SINK_EXPORT SinkResource : public ApplicationDomainType {
+    static constexpr const char *name = "resource";
     typedef QSharedPointer<SinkResource> Ptr;
     explicit SinkResource(const QByteArray &resourceInstanceIdentifier, const QByteArray &identifier, qint64 revision, const QSharedPointer<BufferAdaptor> &adaptor);
     explicit SinkResource(const QByteArray &identifier);
@@ -300,8 +343,6 @@ struct SINK_EXPORT SinkResource : public ApplicationDomainType {
     SINK_PROPERTY(QByteArray, ResourceType, type);
     SINK_PROPERTY(QByteArrayList, Capabilities, capabilities);
     SINK_STATUS_PROPERTY(int, Status, status);
-    SINK_STATUS_PROPERTY(ApplicationDomain::Error, Error, error);
-    SINK_STATUS_PROPERTY(ApplicationDomain::Progress, Progress, progress);
 };
 
 struct SINK_EXPORT Entity : public ApplicationDomainType {
@@ -312,16 +353,35 @@ struct SINK_EXPORT Entity : public ApplicationDomainType {
     virtual ~Entity() = default;
 };
 
+struct SINK_EXPORT Addressbook : public Entity {
+    SINK_ENTITY(Addressbook, addressbook);
+    SINK_REFERENCE_PROPERTY(Addressbook, Parent, parent);
+    SINK_PROPERTY(QString, Name, name);
+    SINK_EXTRACTED_PROPERTY(QDateTime, LastUpdated, lastUpdated);
+};
+
 struct SINK_EXPORT Contact : public Entity {
-    SINK_ENTITY(Contact);
+    struct SINK_EXPORT Email {
+        enum Type {
+            Undefined,
+            Work,
+            Home
+        };
+        Type type;
+        QString email;
+    };
+    SINK_ENTITY(Contact, contact);
     SINK_PROPERTY(QString, Uid, uid);
     SINK_PROPERTY(QString, Fn, fn);
-    SINK_PROPERTY(QByteArrayList, Emails, emails);
+    SINK_PROPERTY(QString, Firstname, firstname);
+    SINK_PROPERTY(QString, Lastname, lastname);
+    SINK_PROPERTY(QList<Email>, Emails, emails);
     SINK_PROPERTY(QByteArray, Vcard, vcard);
+    SINK_REFERENCE_PROPERTY(Addressbook, Addressbook, addressbook);
 };
 
 struct SINK_EXPORT Event : public Entity {
-    SINK_ENTITY(Event);
+    SINK_ENTITY(Event, event);
     SINK_PROPERTY(QString, Uid, uid);
     SINK_PROPERTY(QString, Summary, summary);
     SINK_PROPERTY(QString, Description, description);
@@ -329,20 +389,23 @@ struct SINK_EXPORT Event : public Entity {
 };
 
 struct SINK_EXPORT Todo : public Entity {
-    SINK_ENTITY(Todo);
+    SINK_ENTITY(Todo, todo);
 };
 
 struct SINK_EXPORT Calendar : public Entity {
-    SINK_ENTITY(Calendar);
+    SINK_ENTITY(Calendar, calendar);
 };
 
 struct SINK_EXPORT Folder : public Entity {
-    SINK_ENTITY(Folder);
+    SINK_ENTITY(Folder, folder);
     SINK_REFERENCE_PROPERTY(Folder, Parent, parent);
     SINK_PROPERTY(QString, Name, name);
     SINK_PROPERTY(QByteArray, Icon, icon);
     SINK_PROPERTY(QByteArrayList, SpecialPurpose, specialpurpose);
     SINK_PROPERTY(bool, Enabled, enabled);
+    SINK_EXTRACTED_PROPERTY(QDateTime, LastUpdated, lastUpdated);
+    SINK_EXTRACTED_PROPERTY(int, Count, count);
+    SINK_EXTRACTED_PROPERTY(bool, FullContentAvailable, fullContentAvailable);
 };
 
 struct SINK_EXPORT Mail : public Entity {
@@ -351,7 +414,7 @@ struct SINK_EXPORT Mail : public Entity {
         QString emailAddress;
     };
 
-    SINK_ENTITY(Mail);
+    SINK_ENTITY(Mail, mail);
     SINK_EXTRACTED_PROPERTY(Contact, Sender, sender);
     SINK_EXTRACTED_PROPERTY(QList<Contact>, To, to);
     SINK_EXTRACTED_PROPERTY(QList<Contact>, Cc, cc);
@@ -373,23 +436,8 @@ struct SINK_EXPORT Mail : public Entity {
 
 SINK_EXPORT QDebug operator<< (QDebug d, const Mail::Contact &c);
 
-/**
- * The status of an account or resource.
- *
- * It is set as follows:
- * * By default the status is offline.
- * * If a connection to the server could be established the status is Connected.
- * * If an error occurred that keeps the resource from operating (so non transient), the resource enters the error state.
- * * If a long running operation is started the resource goes to the busy state (and return to the previous state after that).
- */
-enum SINK_EXPORT Status {
-    OfflineStatus,
-    ConnectedStatus,
-    BusyStatus,
-    ErrorStatus
-};
-
 struct SINK_EXPORT Identity : public ApplicationDomainType {
+    static constexpr const char *name = "identity";
     typedef QSharedPointer<Identity> Ptr;
     explicit Identity(const QByteArray &resourceInstanceIdentifier, const QByteArray &identifier, qint64 revision, const QSharedPointer<BufferAdaptor> &adaptor);
     explicit Identity(const QByteArray &identifier);
@@ -416,6 +464,10 @@ struct SINK_EXPORT ImapResource {
     static SinkResource create(const QByteArray &account);
 };
 
+struct SINK_EXPORT CardDavResource {
+    static SinkResource create(const QByteArray &account);
+};
+
 namespace ResourceCapabilities {
 namespace Mail {
     static constexpr const char *mail = "mail";
@@ -427,6 +479,11 @@ namespace Mail {
     static constexpr const char *transport = "mail.transport";
     static constexpr const char *folderhierarchy = "mail.folderhierarchy";
 };
+namespace Contact {
+    static constexpr const char *contact = "contact";
+    static constexpr const char *addressbook = "addressbook";
+    static constexpr const char *storage = "contact.storage";
+};
 };
 
 namespace SpecialPurpose {
@@ -444,31 +501,10 @@ namespace Mail {
  * Do not store these types to disk, they may change over time.
  */
 template<class DomainType>
-QByteArray SINK_EXPORT getTypeName();
-
-template<>
-QByteArray SINK_EXPORT getTypeName<Contact>();
-
-template<>
-QByteArray SINK_EXPORT getTypeName<Event>();
-
-template<>
-QByteArray SINK_EXPORT getTypeName<Todo>();
-
-template<>
-QByteArray SINK_EXPORT getTypeName<SinkResource>();
-
-template<>
-QByteArray SINK_EXPORT getTypeName<SinkAccount>();
-
-template<>
-QByteArray SINK_EXPORT getTypeName<Identity>();
-
-template<>
-QByteArray SINK_EXPORT getTypeName<Mail>();
-
-template<>
-QByteArray SINK_EXPORT getTypeName<Folder>();
+QByteArray SINK_EXPORT getTypeName()
+{
+    return DomainType::name;
+}
 
 QByteArrayList SINK_EXPORT getTypeNames();
 
@@ -499,6 +535,7 @@ class SINK_EXPORT TypeImplementation;
  */
 #define SINK_REGISTER_TYPES() \
     REGISTER_TYPE(Sink::ApplicationDomain::Contact) \
+    REGISTER_TYPE(Sink::ApplicationDomain::Addressbook) \
     REGISTER_TYPE(Sink::ApplicationDomain::Event) \
     REGISTER_TYPE(Sink::ApplicationDomain::Mail) \
     REGISTER_TYPE(Sink::ApplicationDomain::Folder) \
@@ -520,6 +557,7 @@ Q_DECLARE_METATYPE(Sink::ApplicationDomain::ApplicationDomainType::Ptr)
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::Entity)
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::Entity::Ptr)
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::Mail::Contact)
+Q_DECLARE_METATYPE(Sink::ApplicationDomain::Contact::Email)
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::Error)
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::Progress)
 Q_DECLARE_METATYPE(Sink::ApplicationDomain::BLOB)
