@@ -547,6 +547,7 @@ static Sink::Notification getNotification(const Sink::Commands::Notification *bu
     }
     n.type = buffer->type();
     n.code = buffer->code();
+    n.entities = BufferUtils::fromVector(*buffer->entities());
     return n;
 }
 
@@ -601,20 +602,23 @@ bool ResourceAccess::processMessageBuffer()
                     queuedInvoke([=]() { emit notification(n); }, this);
                 } break;
                 case Sink::Notification::Status:
-                    if (mResourceStatus == buffer->code()) {
-                        SinkTrace() << "Got an unnecessary status notification";
-                        break;
+                    if (mResourceStatus != buffer->code()) {
+                        mResourceStatus = buffer->code();
+                        SinkTrace() << "Updated status: " << mResourceStatus;
                     }
-                    mResourceStatus = buffer->code();
-                    SinkTrace() << "Updated status: " << mResourceStatus;
+                    [[clang::fallthrough]];
+                case Sink::Notification::Info:
                     [[clang::fallthrough]];
                 case Sink::Notification::Warning:
                     [[clang::fallthrough]];
+                case Sink::Notification::Error:
+                    [[clang::fallthrough]];
                 case Sink::Notification::FlushCompletion:
                     [[clang::fallthrough]];
                 case Sink::Notification::Progress: {
                     auto n = getNotification(buffer);
-                    SinkTrace() << "Received notification: " << n.type;
+                    SinkTrace() << "Received notification: " << n;
+                    n.resource = d->resourceInstanceIdentifier;
                     emit notification(n);
                 } break;
                 case Sink::Notification::RevisionUpdate:
