@@ -62,6 +62,44 @@ int getErrorCode(int e)
     return -1;
 }
 
+static QList<QByteArray> getDatabaseNames(MDB_txn *transaction)
+{
+    if (!transaction) {
+        SinkWarning() << "Invalid transaction";
+        return QList<QByteArray>();
+    }
+    int rc;
+    QList<QByteArray> list;
+    MDB_dbi dbi;
+    if ((rc = mdb_dbi_open(transaction, nullptr, 0, &dbi) == 0)) {
+        MDB_val key;
+        MDB_val data;
+        MDB_cursor *cursor;
+
+        mdb_cursor_open(transaction, dbi, &cursor);
+        if ((rc = mdb_cursor_get(cursor, &key, &data, MDB_FIRST)) == 0) {
+            list << QByteArray::fromRawData((char *)key.mv_data, key.mv_size);
+            while ((rc = mdb_cursor_get(cursor, &key, &data, MDB_NEXT)) == 0) {
+                list << QByteArray::fromRawData((char *)key.mv_data, key.mv_size);
+            }
+        } else {
+            //Normal if we don't have any databases yet
+            if (rc == MDB_NOTFOUND) {
+                rc = 0;
+            }
+            if (rc) {
+                SinkWarning() << "Failed to get a value" << rc;
+            }
+        }
+        mdb_cursor_close(cursor);
+    } else {
+        SinkWarning() << "Failed to open db" << rc << QByteArray(mdb_strerror(rc));
+    }
+    return list;
+
+}
+
+
 class DataStore::NamedDatabase::Private
 {
 public:
@@ -93,6 +131,18 @@ public:
         const auto dbiName = name + db;
         if (sDbis.contains(dbiName)) {
             dbi = sDbis.value(dbiName);
+            //sDbis can contain dbi's that are not available to this transaction.
+            //We use mdb_dbi_flags to check if the dbi is valid for this transaction.
+            uint f;
+            if (mdb_dbi_flags(transaction, dbi, &f) == EINVAL) {
+                //In readonly mode we can just ignore this. In read-write we would have tried to concurrently create a db.
+                if (!readOnly) {
+                    SinkWarning() << "Tried to create database in second transaction: " << dbiName;
+                }
+                dbi = 0;
+                transaction = 0;
+                return false;
+            }
         } else {
             MDB_dbi flagtableDbi;
             if (const int rc = mdb_dbi_open(transaction, "__flagtable", readOnly ? 0 : MDB_CREATE, &flagtableDbi)) {
@@ -279,7 +329,8 @@ int DataStore::NamedDatabase::scan(const QByteArray &k, const std::function<bool
 
     rc = mdb_cursor_open(d->transaction, d->dbi, &cursor);
     if (rc) {
-        Error error(d->name.toLatin1() + d->db, getErrorCode(rc), QByteArray("Error during mdb_cursor open: ") + QByteArray(mdb_strerror(rc)));
+        //Invalid arguments can mean that the transaction doesn't contain the db dbi
+        Error error(d->name.toLatin1() + d->db, getErrorCode(rc), QByteArray("Error during mdb_cursor_open: ") + QByteArray(mdb_strerror(rc)) + ". Key: " + k);
         errorHandler ? errorHandler(error) : d->defaultErrorHandler(error);
         return 0;
     }
@@ -447,7 +498,6 @@ public:
 
     MDB_env *env;
     MDB_txn *transaction;
-    MDB_dbi dbi;
     bool requestedRead;
     std::function<void(const DataStore::Error &error)> defaultErrorHandler;
     QString name;
@@ -578,8 +628,7 @@ static bool ensureCorrectDb(DataStore::NamedDatabase &database, const QByteArray
         }
         return false;
     },
-    [](const DataStore::Error &error) -> bool{
-        return false;
+    [&](const DataStore::Error &) {
     }, false);
     //This is the first time we open this database in a write transaction, write the db name
     if (!count) {
@@ -637,35 +686,8 @@ QList<QByteArray> DataStore::Transaction::getDatabaseNames() const
         SinkWarning() << "Invalid transaction";
         return QList<QByteArray>();
     }
+    return Sink::Storage::getDatabaseNames(d->transaction);
 
-    int rc;
-    QList<QByteArray> list;
-    Q_ASSERT(d->transaction);
-    if ((rc = mdb_dbi_open(d->transaction, nullptr, 0, &d->dbi) == 0)) {
-        MDB_val key;
-        MDB_val data;
-        MDB_cursor *cursor;
-
-        mdb_cursor_open(d->transaction, d->dbi, &cursor);
-        if ((rc = mdb_cursor_get(cursor, &key, &data, MDB_FIRST)) == 0) {
-            list << QByteArray::fromRawData((char *)key.mv_data, key.mv_size);
-            while ((rc = mdb_cursor_get(cursor, &key, &data, MDB_NEXT)) == 0) {
-                list << QByteArray::fromRawData((char *)key.mv_data, key.mv_size);
-            }
-        } else {
-            //Normal if we don't have any databases yet
-            if (rc == MDB_NOTFOUND) {
-                rc = 0;
-            }
-            if (rc) {
-                SinkWarning() << "Failed to get a value" << rc;
-            }
-        }
-        mdb_cursor_close(cursor);
-    } else {
-        SinkWarning() << "Failed to open db" << rc << QByteArray(mdb_strerror(rc));
-    }
-    return list;
 }
 
 
