@@ -92,6 +92,7 @@ QPair<typename AggregatingResultEmitter<typename DomainType::Ptr>::Ptr,  typenam
         auto facade = FacadeFactory::instance().getFacade<ApplicationDomain::SinkResource>();
         Q_ASSERT(facade);
         Sink::Query resourceQuery;
+        resourceQuery.request<ApplicationDomain::SinkResource::Capabilities>();
         if (query.liveQuery()) {
             SinkTraceCtx(ctx) << "Listening for new resources.";
             resourceQuery.setFlags(Query::LiveQuery);
@@ -103,6 +104,7 @@ QPair<typename AggregatingResultEmitter<typename DomainType::Ptr>::Ptr,  typenam
             resourceFilter.propertyFilter.insert(ApplicationDomain::SinkResource::Capabilities::name, Query::Comparator{ApplicationDomain::getTypeName<DomainType>(), Query::Comparator::Contains});
         }
         resourceQuery.setFilter(resourceFilter);
+        resourceQuery.requestedProperties << resourceFilter.propertyFilter.keys();
 
         auto result = facade->load(resourceQuery, resourceCtx);
         auto emitter = result.second;
@@ -249,7 +251,7 @@ KAsync::Job<void> Store::remove(const Sink::Query &query)
 KAsync::Job<void> Store::removeDataFromDisk(const QByteArray &identifier)
 {
     // All databases are going to become invalid, nuke the environments
-    // TODO: all clients should react to a notification the resource
+    // TODO: all clients should react to a notification from the resource
     Sink::Storage::DataStore::clearEnv();
     SinkTrace() << "Remove data from disk " << identifier;
     auto time = QSharedPointer<QTime>::create();
@@ -277,18 +279,18 @@ KAsync::Job<void> Store::removeDataFromDisk(const QByteArray &identifier)
 
 static KAsync::Job<void> synchronize(const QByteArray &resource, const Sink::SyncScope &scope)
 {
-    SinkLog() << "Synchronizing " << resource;
+    SinkLog() << "Synchronizing " << resource << scope;
     auto resourceAccess = ResourceAccessFactory::instance().getAccess(resource, ResourceConfig::getResourceType(resource));
     return resourceAccess->synchronizeResource(scope)
         .addToContext(resourceAccess)
-        .then<void>([](const KAsync::Error &error) {
-                if (error) {
-                    SinkWarning() << "Error during sync.";
-                    return KAsync::error<void>(error);
-                }
-                SinkTrace() << "synced.";
-                return KAsync::null<void>();
-            });
+        .then([=](const KAsync::Error &error) {
+            if (error) {
+                SinkWarning() << "Error during sync.";
+                return KAsync::error(error);
+            }
+            SinkTrace() << "Synchronization of resource " << resource << " complete.";
+            return KAsync::null();
+        });
 }
 
 KAsync::Job<void> Store::synchronize(const Sink::Query &query)
