@@ -100,7 +100,7 @@ public:
     QByteArray createFolder(const Imap::Folder &f)
     {
         const auto parentFolderRid = parentRid(f);
-        SinkTrace() << "Creating folder: " << f.name() << parentFolderRid;
+        SinkTraceCtx(mLogCtx) << "Creating folder: " << f.name() << parentFolderRid;
 
         const auto remoteId = folderRid(f);
         Sink::ApplicationDomain::Folder folder;
@@ -123,7 +123,7 @@ public:
 
     void synchronizeFolders(const QVector<Folder> &folderList)
     {
-        SinkTrace() << "Found folders " << folderList.size();
+        SinkTraceCtx(mLogCtx) << "Found folders " << folderList.size();
 
         scanForRemovals(ENTITY_TYPE_FOLDER,
             [&folderList](const QByteArray &remoteId) -> bool {
@@ -164,14 +164,14 @@ public:
     {
         auto time = QSharedPointer<QTime>::create();
         time->start();
-        SinkTrace() << "Importing new mail." << folderRid;
+        SinkTraceCtx(mLogCtx) << "Importing new mail." << folderRid;
 
         const auto folderLocalId = syncStore().resolveRemoteId(ENTITY_TYPE_FOLDER, folderRid);
 
         const auto remoteId = assembleMailRid(folderLocalId, message.uid);
 
         Q_ASSERT(message.msg);
-        SinkTrace() << "Found a mail " << remoteId << message.msg->subject(true)->asUnicodeString() << message.flags;
+        SinkTraceCtx(mLogCtx) << "Found a mail " << remoteId << message.msg->subject(true)->asUnicodeString() << message.flags;
 
         auto mail = Sink::ApplicationDomain::Mail::create(mResourceInstanceIdentifier);
         mail.setFolder(folderLocalId);
@@ -181,7 +181,7 @@ public:
 
         createOrModify(ENTITY_TYPE_MAIL, remoteId, mail);
         // const auto elapsed = time->elapsed();
-        // SinkTrace() << "Synchronized " << count << " mails in " << folderRid << Sink::Log::TraceTime(elapsed) << " " << elapsed/qMax(count, 1) << " [ms/mail]";
+        // SinkTraceCtx(mLogCtx) << "Synchronized " << count << " mails in " << folderRid << Sink::Log::TraceTime(elapsed) << " " << elapsed/qMax(count, 1) << " [ms/mail]";
     }
 
     void synchronizeRemovals(const QByteArray &folderRid, const QSet<qint64> &messages)
@@ -194,7 +194,7 @@ public:
             return;
         }
 
-        SinkTrace() << "Finding removed mail: " << folderLocalId << " remoteId: " << folderRid;
+        SinkTraceCtx(mLogCtx) << "Finding removed mail: " << folderLocalId << " remoteId: " << folderRid;
 
         int count = 0;
 
@@ -370,7 +370,7 @@ public:
 
     Sink::QueryBase applyMailDefaults(const Sink::QueryBase &query)
     {
-        auto defaultDateFilter = QDate::currentDate().addDays(-14);
+        auto defaultDateFilter = QDate::currentDate().addDays(0 - mDaysToSync);
         auto queryWithDefaults = query;
         if (!queryWithDefaults.hasFilter<ApplicationDomain::Mail::Date>()) {
             queryWithDefaults.filter(ApplicationDomain::Mail::Date::name, QVariant::fromValue(defaultDateFilter));
@@ -382,7 +382,11 @@ public:
     {
         QList<Synchronizer::SyncRequest> list;
         if (query.type() == ApplicationDomain::getTypeName<ApplicationDomain::Mail>()) {
-            list << Synchronizer::SyncRequest{applyMailDefaults(query)};
+            auto request = Synchronizer::SyncRequest{applyMailDefaults(query)};
+            if (query.hasFilter(ApplicationDomain::Mail::Folder::name)) {
+                request.applicableEntities << query.getFilter(ApplicationDomain::Mail::Folder::name).value.toByteArray();
+            }
+            list << request;
         } else if (query.type() == ApplicationDomain::getTypeName<ApplicationDomain::Folder>()) {
             list << Synchronizer::SyncRequest{query};
         } else {
@@ -393,15 +397,57 @@ public:
         return list;
     }
 
+    QByteArray getFolderFromLocalId(const QByteArray &id)
+    {
+        auto mailRemoteId = syncStore().resolveLocalId(ApplicationDomain::getTypeName<ApplicationDomain::Mail>(), id);
+        if (mailRemoteId.isEmpty()) {
+            return {};
+        }
+        return folderIdFromMailRid(mailRemoteId);
+    }
+
+    void mergeIntoQueue(const Synchronizer::SyncRequest &request, QList<Synchronizer::SyncRequest> &queue)  Q_DECL_OVERRIDE
+    {
+        auto isIndividualMailSync = [](const Synchronizer::SyncRequest &request) {
+            if (request.requestType == SyncRequest::Synchronization) {
+                const auto query = request.query;
+                if (query.type() == ApplicationDomain::getTypeName<ApplicationDomain::Mail>()) {
+                    return !query.ids().isEmpty();
+                }
+            }
+            return false;
+
+        };
+
+        if (isIndividualMailSync(request)) {
+            auto newId = request.query.ids().first();
+            auto requestFolder = getFolderFromLocalId(newId);
+            if (requestFolder.isEmpty()) {
+                SinkWarningCtx(mLogCtx) << "Failed to find folder for local id. Ignoring request: " << request.query;
+                return;
+            }
+            for (auto &r : queue) {
+                if (isIndividualMailSync(r)) {
+                    auto queueFolder = getFolderFromLocalId(r.query.ids().first());
+                    if (requestFolder == queueFolder) {
+                        //Merge
+                        r.query.filter(newId);
+                        SinkTrace() << "Merging request " << request.query;
+                        SinkTrace() << " to " << r.query;
+                        return;
+                    }
+                }
+            }
+        }
+        queue << request;
+    }
+
     KAsync::Job<void> login(QSharedPointer<ImapServerProxy> imap)
     {
         SinkTrace() << "Connecting to:" << mServer << mPort;
         SinkTrace() << "as:" << mUser;
         return imap->login(mUser, mPassword)
-        .addToContext(imap)
-        .onError([](const KAsync::Error &error) {
-            SinkWarning() << "Login failed.";
-        });
+        .addToContext(imap);
     }
 
     KAsync::Job<QVector<Folder>> getFolderList(QSharedPointer<ImapServerProxy> imap, const Sink::QueryBase &query)
@@ -431,6 +477,19 @@ public:
         }
     }
 
+    KAsync::Error getError(const KAsync::Error &error)
+    {
+        if (error) {
+            if (error.errorCode == Imap::CouldNotConnectError) {
+                return {ApplicationDomain::ConnectionError, error.errorMessage};
+            } else if (error.errorCode == Imap::SslHandshakeError) {
+                return {ApplicationDomain::LoginError, error.errorMessage};
+            }
+            return {ApplicationDomain::UnknownError, error.errorMessage};
+        }
+        return {};
+    }
+
     KAsync::Job<void> synchronizeWithSource(const Sink::QueryBase &query) Q_DECL_OVERRIDE
     {
         auto imap = QSharedPointer<ImapServerProxy>::create(mServer, mPort, &mSessionCache);
@@ -446,11 +505,8 @@ public:
                 });
             })
             .then([=] (const KAsync::Error &error) {
-                if (error) {
-                    SinkWarning() << "Error during folder sync: " << error.errorMessage;
-                }
                 return imap->logout()
-                    .then(KAsync::error(error));
+                    .then(KAsync::error(getError(error)));
             });
         } else if (query.type() == ApplicationDomain::getTypeName<ApplicationDomain::Mail>()) {
             //TODO
@@ -487,7 +543,7 @@ public:
                         synchronizeMails(folderRemoteId, m);
                     },
                     [=](int progress, int total) {
-                        SinkLogCtx(mLogCtx) << "Progress: " << progress << " out of " << total;
+                        reportProgress(progress, total);
                         //commit every 100 messages
                         if ((progress % 100) == 0) {
                             commit();
@@ -503,6 +559,8 @@ public:
                         return KAsync::value(folders)
                         .serialEach<void>([=](const Folder &folder) {
                             SinkLog() << "Syncing folder " << folder.path();
+                            //Emit notification that the folder is being synced.
+                            //The synchronizer can't do that because it has no concept of the folder filter on a mail sync scope meaning that the folder is being synchronized.
                             QDate dateFilter;
                             auto filter = query.getFilter<ApplicationDomain::Mail::Date>();
                             if (filter.value.canConvert<QDate>()) {
@@ -510,7 +568,7 @@ public:
                                 SinkLog() << " with date-range " << dateFilter;
                             }
                             return synchronizeFolder(imap, folder, dateFilter, syncHeaders)
-                                .onError([folder](const KAsync::Error &error) {
+                                .onError([=](const KAsync::Error &error) {
                                     SinkWarning() << "Failed to sync folder: " << folder.path() << "Error: " << error.errorMessage;
                                 });
                         });
@@ -518,11 +576,8 @@ public:
                 }
             })
             .then([=] (const KAsync::Error &error) {
-                if (error) {
-                    SinkWarning() << "Error during sync: " << error.errorMessage;
-                }
                 return imap->logout()
-                    .then(KAsync::error(error));
+                    .then(KAsync::error(getError(error)));
             });
         }
         return KAsync::error<void>("Nothing to do");
@@ -616,11 +671,11 @@ public:
             if (!folder.getParent().isEmpty()) {
                 parentFolder = syncStore().resolveLocalId(ENTITY_TYPE_FOLDER, folder.getParent());
             }
-            SinkTrace() << "Creating a new folder: " << parentFolder << folder.getName();
+            SinkTraceCtx(mLogCtx) << "Creating a new folder: " << parentFolder << folder.getName();
             auto rid = QSharedPointer<QByteArray>::create();
             auto createFolder = login.then(imap->createSubfolder(parentFolder, folder.getName()))
-                .then([imap, rid](const QString &createdFolder) {
-                    SinkTrace() << "Finished creating a new folder: " << createdFolder;
+                .then([this, imap, rid](const QString &createdFolder) {
+                    SinkTraceCtx(mLogCtx) << "Finished creating a new folder: " << createdFolder;
                     *rid = createdFolder.toUtf8();
                 });
             if (folder.getSpecialPurpose().isEmpty()) {
@@ -636,19 +691,19 @@ public:
                             specialPurposeFolders->insert(SpecialPurpose::getSpecialPurposeType(folder.name()), folder.path());
                         };
                     }))
-                    .then([specialPurposeFolders, folder, imap, parentFolder, rid]() -> KAsync::Job<void> {
+                    .then([this, specialPurposeFolders, folder, imap, parentFolder, rid]() -> KAsync::Job<void> {
                         for (const auto &purpose : folder.getSpecialPurpose()) {
                             if (specialPurposeFolders->contains(purpose)) {
                                 auto f = specialPurposeFolders->value(purpose);
-                                SinkTrace() << "Merging specialpurpose folder with: " << f << " with purpose: " << purpose;
+                                SinkTraceCtx(mLogCtx) << "Merging specialpurpose folder with: " << f << " with purpose: " << purpose;
                                 *rid = f.toUtf8();
                                 return KAsync::null<void>();
                             }
                         }
-                        SinkTrace() << "No match found for merging, creating a new folder";
+                        SinkTraceCtx(mLogCtx) << "No match found for merging, creating a new folder";
                         return imap->createSubfolder(parentFolder, folder.getName())
-                            .then([imap, rid](const QString &createdFolder) {
-                                SinkTrace() << "Finished creating a new folder: " << createdFolder;
+                            .then([this, imap, rid](const QString &createdFolder) {
+                                SinkTraceCtx(mLogCtx) << "Finished creating a new folder: " << createdFolder;
                                 *rid = createdFolder.toUtf8();
                             });
 
@@ -659,18 +714,18 @@ public:
                 return mergeJob;
             }
         } else if (operation == Sink::Operation_Removal) {
-            SinkTrace() << "Removing a folder: " << oldRemoteId;
+            SinkTraceCtx(mLogCtx) << "Removing a folder: " << oldRemoteId;
             return login.then(imap->remove(oldRemoteId))
-                .then([oldRemoteId, imap] {
-                    SinkTrace() << "Finished removing a folder: " << oldRemoteId;
+                .then([this, oldRemoteId, imap] {
+                    SinkTraceCtx(mLogCtx) << "Finished removing a folder: " << oldRemoteId;
                     return QByteArray();
                 });
         } else if (operation == Sink::Operation_Modification) {
-            SinkTrace() << "Renaming a folder: " << oldRemoteId << folder.getName();
+            SinkTraceCtx(mLogCtx) << "Renaming a folder: " << oldRemoteId << folder.getName();
             auto rid = QSharedPointer<QByteArray>::create();
             return login.then(imap->renameSubfolder(oldRemoteId, folder.getName()))
-                .then([imap, rid](const QString &createdFolder) {
-                    SinkTrace() << "Finished renaming a folder: " << createdFolder;
+                .then([this, imap, rid](const QString &createdFolder) {
+                    SinkTraceCtx(mLogCtx) << "Finished renaming a folder: " << createdFolder;
                     *rid = createdFolder.toUtf8();
                 })
                 .then([rid] {
@@ -685,6 +740,7 @@ public:
     int mPort;
     QString mUser;
     QString mPassword;
+    int mDaysToSync = 0;
     QByteArray mResourceInstanceIdentifier;
     Imap::SessionCache mSessionCache;
 };
@@ -864,6 +920,7 @@ ImapResource::ImapResource(const ResourceContext &resourceContext)
     synchronizer->mPort = port;
     synchronizer->mUser = user;
     synchronizer->mPassword = password;
+    synchronizer->mDaysToSync = 14;
     setupSynchronizer(synchronizer);
 
     auto inspector = QSharedPointer<ImapInspector>::create(resourceContext);
@@ -873,7 +930,7 @@ ImapResource::ImapResource(const ResourceContext &resourceContext)
     inspector->mPassword = password;
     setupInspector(inspector);
 
-    setupPreprocessors(ENTITY_TYPE_MAIL, QVector<Sink::Preprocessor*>() << new SpecialPurposeProcessor(resourceContext.resourceType, resourceContext.instanceId()) << new MailPropertyExtractor);
+    setupPreprocessors(ENTITY_TYPE_MAIL, QVector<Sink::Preprocessor*>() << new SpecialPurposeProcessor << new MailPropertyExtractor);
     setupPreprocessors(ENTITY_TYPE_FOLDER, QVector<Sink::Preprocessor*>());
 }
 
@@ -898,14 +955,14 @@ Sink::Resource *ImapResourceFactory::createResource(const ResourceContext &conte
 
 void ImapResourceFactory::registerFacades(const QByteArray &name, Sink::FacadeFactory &factory)
 {
-    factory.registerFacade<Sink::ApplicationDomain::Mail, ImapResourceMailFacade>(name);
-    factory.registerFacade<Sink::ApplicationDomain::Folder, ImapResourceFolderFacade>(name);
+    factory.registerFacade<ApplicationDomain::Mail, DefaultFacade<ApplicationDomain::Mail>>(name);
+    factory.registerFacade<ApplicationDomain::Folder, DefaultFacade<ApplicationDomain::Folder>>(name);
 }
 
 void ImapResourceFactory::registerAdaptorFactories(const QByteArray &name, Sink::AdaptorFactoryRegistry &registry)
 {
-    registry.registerFactory<Sink::ApplicationDomain::Mail, ImapMailAdaptorFactory>(name);
-    registry.registerFactory<Sink::ApplicationDomain::Folder, ImapFolderAdaptorFactory>(name);
+    registry.registerFactory<ApplicationDomain::Mail, DefaultAdaptorFactory<ApplicationDomain::Mail>>(name);
+    registry.registerFactory<ApplicationDomain::Folder, DefaultAdaptorFactory<ApplicationDomain::Folder>>(name);
 }
 
 void ImapResourceFactory::removeDataFromDisk(const QByteArray &instanceIdentifier)
