@@ -69,7 +69,7 @@ static KAsync::Job<T> runJob(KJob *job, const std::function<T(KJob*)> &f)
         QObject::connect(job, &KJob::result, [&future, f](KJob *job) {
             SinkTrace() << "Job done: " << job->metaObject()->className();
             if (job->error()) {
-                SinkWarning() << "Job failed: " << job->errorString();
+                SinkWarning() << "Job failed: " << job->errorString() << job->metaObject()->className();
                 future.setError(job->error(), job->errorString());
             } else {
                 future.setValue(f(job));
@@ -87,7 +87,7 @@ static KAsync::Job<void> runJob(KJob *job)
         QObject::connect(job, &KJob::result, [&future](KJob *job) {
             SinkTrace() << "Job done: " << job->metaObject()->className();
             if (job->error()) {
-                SinkWarning() << "Job failed: " << job->errorString();
+                SinkWarning() << "Job failed: " << job->errorString() << job->metaObject()->className();
                 future.setError(job->error(), job->errorString());
             } else {
                 future.setFinished();
@@ -159,6 +159,16 @@ KAsync::Job<void> ImapServerProxy::login(const QString &username, const QString
         // SinkTrace() << "Found personal namespaces: " << mNamespaces.personal;
         // SinkTrace() << "Found shared namespaces: " << mNamespaces.shared;
         // SinkTrace() << "Found user namespaces: " << mNamespaces.user;
+    }).then([=] (const KAsync::Error &error) {
+        if (error) {
+            if (error.errorCode == KIMAP2::LoginJob::ErrorCode::ERR_COULD_NOT_CONNECT) {
+                return KAsync::error(CouldNotConnectError, "Failed to connect: " + error.errorMessage);
+            } else if (error.errorCode == KIMAP2::LoginJob::ErrorCode::ERR_SSL_HANDSHAKE_FAILED) {
+                return KAsync::error(SslHandshakeError, "Ssl handshake failed: " + error.errorMessage);
+            }
+            return KAsync::error(error);
+        }
+        return KAsync::null();
     });
 }
 
